/* Copyright (c) 2014 ScriptDev2_R2 <http://github.com/mangosR2/scriptdev2/>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* ScriptData
SDName: holiday_scripts.cpp
SD%Complete: 100
SDComment: Quest support: 24849, 24851
SDCategory: Holidays
EndScriptData */

/* ContentData
at_hot_on_the_trail
npc_snivel_rustrocket
EndContentData */

#include "precompiled.h"
#include "GameEventMgr.h"
#include "escort_ai.h"

/*****************************************/
/**********  Love is in the Air **********/
/*****************************************/

/*######
## at_hot_on_the_trail
######*/

enum
{
    QUEST_HOT_ON_THE_TRAIL_A = 24849,
    QUEST_HOT_ON_THE_TRAIL_H = 24851,

    AT_BANK_A                = 5710,
    AT_AH_A                  = 5711,
    AT_BARBER_A              = 5712,
    AT_BANK_H                = 5715,
    AT_AH_H                  = 5714,
    AT_BARBER_H              = 5716,

    NPC_BUNNY_BANK           = 38340,
    NPC_BUNNY_AH             = 38341,
    NPC_BUNNY_BARBER         = 38342,

    SPELL_BANK_A             = 71713,
    SPELL_AH_A               = 71745,
    SPELL_BARBER_A           = 71752,
    SPELL_BANK_H             = 71759,
    SPELL_AH_H               = 71760,
    SPELL_BARBER_H           = 71758,

    AURA_BANK_A              = 71712,
    AURA_AH_A                = 71744,
    AURA_BARBER_A            = 71751,
    AURA_BANK_H              = 71762,
    AURA_AH_H                = 71763,
    AURA_BARBER_H            = 71765,
};

bool AreaTrigger_at_hot_on_the_trail(Player* pPlayer, AreaTriggerEntry const* pAt)
{
    if (!pPlayer->isAlive() ||
        (pPlayer->GetQuestStatus(QUEST_HOT_ON_THE_TRAIL_A) != QUEST_STATUS_INCOMPLETE &&
        pPlayer->GetQuestStatus(QUEST_HOT_ON_THE_TRAIL_H) != QUEST_STATUS_INCOMPLETE) ||
        !IsHolidayActive(HOLIDAY_LOVE_IS_IN_THE_AIR_NEW))
    {
        return false;
    }

    uint32 bunnyId;
    switch (pAt->id)
    {
        case AT_BANK_A:
        case AT_BANK_H:
            bunnyId = NPC_BUNNY_BANK;
            break;
        case AT_AH_A:
        case AT_AH_H:
            bunnyId = NPC_BUNNY_AH;
            break;
        case AT_BARBER_A:
        case AT_BARBER_H:
            bunnyId = NPC_BUNNY_BARBER;
            break;
        default:
            return false;
    }

    Unit* pBunny = GetClosestCreatureWithEntry(pPlayer, bunnyId, 20.0f, true, false);
    if (!pBunny)
        return false;

    uint32 spellId;
    uint32 auraId;
    switch (pAt->id)
    {
        case AT_BANK_A:   spellId = SPELL_BANK_A;   auraId = AURA_BANK_A;   break;
        case AT_AH_A:     spellId = SPELL_AH_A;     auraId = AURA_AH_A;     break;
        case AT_BARBER_A: spellId = SPELL_BARBER_A; auraId = AURA_BARBER_A; break;
        case AT_BANK_H:   spellId = SPELL_BANK_H;   auraId = AURA_BANK_H;   break;
        case AT_AH_H:     spellId = SPELL_AH_H;     auraId = AURA_AH_H;     break;
        case AT_BARBER_H: spellId = SPELL_BARBER_H; auraId = AURA_BARBER_H; break;
    }

    if (!pPlayer->HasAura(auraId, EFFECT_INDEX_0))
    {
        pBunny->KillSelf();
        pPlayer->CastSpell(pPlayer, spellId, false);
    }

    return false;
}

/*#####
# npc_snivel_rustrocket
#####*/

enum
{
    SAY_BANK_1          = -1020600,
    SAY_BANK_2          = -1020601,
    SAY_AH_1            = -1020602,
    SAY_AH_2            = -1020603,
    SAY_AH_3            = -1020604,
    SAY_BARBER_1        = -1020605,
    SAY_BARBER_2        = -1020606,
    SAY_BARBER_3        = -1020607,
    SAY_BARBER_4        = -1020608,

    NPC_SNIVEL_BANK_A   = 38334,
    NPC_SNIVEL_AH_A     = 38335,
    NPC_SNIVEL_BARBER_A = 38336,
    NPC_SNIVEL_BANK_H   = 38337,
    NPC_SNIVEL_AH_H     = 38338,
    NPC_SNIVEL_BARBER_H = 38339,

    SPELL_ROCKET        = 71715,
    SPELL_BLIND         = 71750,
    SPELL_VANISH        = 24222,

    DLG_BANK_START      = 1,
    DLG_BANK_END        = 2,
    DLG_AH_START        = 3,
    DLG_AH_END          = 4,
    DLG_BARBER_START    = 5,
    DLG_BARBER_VANISH   = 6,
    DLG_BARBER_END      = 7
};

static const DialogueEntry npcSnivelDialogues[] =
{
    // Bank
    {DLG_BANK_START,      0, 2000},
    {SAY_BANK_1,         !0, 3500},
    {SAY_BANK_2,         !0, 2000},
    {DLG_BANK_END,        0,    0},
    // AH
    {DLG_AH_START,        0, 2000},
    {SAY_AH_1,           !0, 3500},
    {SAY_AH_2,           !0, 3500},
    {SAY_AH_3,           !0, 2000},
    {DLG_AH_END,          0,    0},
    // Barber
    {DLG_BARBER_START,    0, 2000},
    {SAY_BARBER_1,       !0, 3500},
    {SAY_BARBER_2,       !0, 3500},
    {SAY_BARBER_3,       !0, 3500},
    {DLG_BARBER_VANISH,   0, 1000},
    {SAY_BARBER_4,       !0,  500},
    {DLG_BARBER_END,      0,    0}
};

struct MANGOS_DLL_DECL npc_snivel_rustrocketAI : public npc_escortAI, private DialogueHelper
{
    npc_snivel_rustrocketAI(Creature* pCreature) :
        npc_escortAI(pCreature), DialogueHelper(npcSnivelDialogues) { Reset(); }

    void Reset() override
    {
        int32 textEntry;
        switch (m_creature->GetEntry())
        {
            case NPC_SNIVEL_BANK_A:
            case NPC_SNIVEL_BANK_H:
                textEntry = EMOTE_ONESHOT_TALK;
                break;
            case NPC_SNIVEL_AH_A:
            case NPC_SNIVEL_AH_H:
                textEntry = SAY_AH_1;
                break;
            case NPC_SNIVEL_BARBER_A:
            case NPC_SNIVEL_BARBER_H:
                textEntry = SAY_BARBER_1;
                break;
            default:
                return;
        }
        StartNextDialogueText(textEntry);
    }

    Creature* GetSpeakerByEntry(uint32 /*uiEntry*/) override
    {
        return m_creature;
    }

    void JustDidDialogueStep(int32 iEntry) override
    {
        switch (iEntry)
        {
            case DLG_BANK_START:
            case DLG_AH_START:
            case DLG_BARBER_START:
                m_creature->HandleEmote(EMOTE_ONESHOT_TALK);
                break;
            case DLG_BANK_END:
                if (Unit* creator = m_creature->GetCreator())
                    m_creature->CastSpell(creator, SPELL_ROCKET, false);
                Start(true);
                break;
            case DLG_AH_END:
                if (Unit* creator = m_creature->GetCreator())
                    m_creature->CastSpell(creator, SPELL_BLIND, false);
                Start(true);
                break;
            case DLG_BARBER_VANISH:
                m_creature->CastSpell(m_creature, SPELL_VANISH, false);
                break;
            case DLG_BARBER_END:
                m_creature->ForcedDespawn(100, false);
                break;
        }
    }

    void JustFinishedEscort(bool& despawn) override
    {
        despawn = false;
        m_creature->ForcedDespawn(100, false);
    }

    void WaypointReached(uint32 /*uiPointId*/) override {}

    void UpdateEscortAI(uint32 const uiDiff) override
    {
        DialogueUpdate(uiDiff);
    }

    static CreatureAI* GetAI(Creature* pCreature)
    {
        return new npc_snivel_rustrocketAI(pCreature);
    }
};

/*######
## Register scripts
######*/

void AddSC_holiday_scripts()
{
    AutoScript s;
    s.newScript("at_hot_on_the_trail")->pAreaTrigger = &AreaTrigger_at_hot_on_the_trail;
    s.newScript("npc_snivel_rustrocket")->GetAI = &npc_snivel_rustrocketAI::GetAI;
}
